' code is Copyright (c) 2008 Julian M Bucknall, www.boyet.com
Public Class SimpleNodePainter
    Implements IBinaryNodePainter
    Implements IDisposable

    Private _surface As Graphics
    Private _edgePen As Pen
    Private _borderPen As Pen
    Private _nodeFill As Brush
    Private _font As Font
    Private _fontBrush As Brush

    Public Sub New(ByVal g As Graphics, ByVal unit As Single)
        _surface = g
        _edgePen = New Pen(Color.Black, 2.0 / unit)
        _borderPen = New Pen(Color.Black, 2.0 / unit)
        _nodeFill = New SolidBrush(Color.SkyBlue)
        _font = New Font("Consolas", 0.5)
        _fontBrush = New SolidBrush(Color.Black)
    End Sub

    Public Sub DrawEdge(ByVal childNumber As Integer, ByVal childLevel As Integer, ByVal parentNumber As Integer) Implements IBinaryNodePainter.DrawEdge
        Dim parentY As Integer = childLevel - 1
        _surface.DrawLine(_edgePen, _
                       New PointF(childNumber, childLevel), _
                       New PointF(parentNumber, parentY) _
                       )
    End Sub

    Public Sub DrawEdgedNode(ByVal node As IBinaryNode, ByVal childNumber As Integer, ByVal childLevel As Integer, ByVal parentNumber As Integer) Implements IBinaryNodePainter.DrawEdgedNode
        DrawEdge(childNumber, childLevel, parentNumber)
        DrawNode(node, childNumber, childLevel)
    End Sub

    Public Sub DrawNode(ByVal node As IBinaryNode, ByVal number As Integer, ByVal level As Integer) Implements IBinaryNodePainter.DrawNode
        Dim nodeRect As RectangleF = New RectangleF(number - 0.5, level - 0.5, 1.0, 1.0)
        _surface.FillEllipse(_nodeFill, nodeRect)
        _surface.DrawEllipse(_borderPen, nodeRect)
        nodeRect.Offset(0.2, 0.15)
        _surface.DrawString(number.ToString(), _font, _fontBrush, nodeRect)
    End Sub

    Private disposedValue As Boolean = False        ' To detect redundant calls

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
                If _edgePen IsNot Nothing Then _edgePen.Dispose()
                If _borderPen IsNot Nothing Then _borderPen.Dispose()
                If _nodeFill IsNot Nothing Then _nodeFill.Dispose()
                If _font IsNot Nothing Then _font.Dispose()
                If _fontBrush IsNot Nothing Then _fontBrush.Dispose()
            End If
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

End Class